import evaluate_model
import os
import numpy as np
import csv
import tools


def evaluate(csv_list, hidden_size=32,
			 noise_ratio_training=0.0, noise_ratio_inference=0.0, repeats=range(10),
			 washout=400, use_active_tuning=False, at_tf_rate=0.0,
			 at_tuning_length=5, at_tuning_cycles=10, at_eta=0.01,
			 at_beta1=0.9, at_beta2=0.999, at_epsilon=1e-8):
	
	# Evaluate the ten models to get the rmse and std for each trained model
	rmse_list = evaluate_model.evaluate(
		hidden_size=hidden_size,
		noise_ratio_training=noise_ratio_training,
		noise_ratio_inference=noise_ratio_inference,
		repeats=repeats,
		washout=washout,
		use_active_tuning=use_active_tuning,
		at_tf_rate=at_tf_rate,
		at_tuning_length=at_tuning_length,
		at_tuning_cycles=at_tuning_cycles,
		at_eta=at_eta,
		at_beta1=at_beta1,
		at_beta2=at_beta2,
		at_epsilon=at_epsilon
	)

	# Create an entry to the csv dictionary for each model run
	for rep in repeats:

		model_name = tools.generate_model_name(
			model_type="LSTM", hidden_size=hidden_size,
			noise_ratio=noise_ratio_training, rep=rep
		)
		csv_list.append({
			"model_name": 			 model_name,
			"noise_ratio_training":  noise_ratio_training,
			"noise_ratio_inference": noise_ratio_inference,
			"use_active_tuning": 	 use_active_tuning,
			"rmse":                  rmse_list[rep]
		})

	# Create an entry for all models in this configuration with mean rmse and
	# std over the rmse of the models
	model_name = tools.generate_model_name(
		model_type="LSTM", hidden_size=hidden_size,
		noise_ratio=noise_ratio_training, rep=19
	)

	rmse_mean = np.mean(rmse_list)
	rmse_std = np.std(rmse_list)


	csv_list.append({
		"model_name": 			 model_name,
		"noise_ratio_training":  noise_ratio_training,
		"noise_ratio_inference": noise_ratio_inference,
		"use_active_tuning": 	 use_active_tuning,
		"rmse":					 rmse_mean,
		"std":					 rmse_std
	})

	return csv_list, rmse_mean, rmse_std


# CSV setup
csv_headers = ["model_name", "noise_ratio_training", "noise_ratio_inference",
			   "use_active_tuning", "rmse", "std"]
csv_list = []

# Initialize the training and inference noise ratios
noise_ratios_training = [0.0, 0.1, 0.2, 0.5, 1.0]
noise_ratios_inference = [0.0, 0.1, 0.2, 0.5, 1.0]

for noise_ratio_inference in noise_ratios_inference:

	print(noise_ratio_inference, end="")

	for noise_ratio_training in noise_ratios_training:

		# Evaluation without active
		csv_list, mean, std = evaluate(
			csv_list=csv_list,
			hidden_size=32,
			noise_ratio_training=noise_ratio_training,
			noise_ratio_inference=noise_ratio_inference,
		)

		blob = " & $" + '{:.4f}'.format(mean) + "$"
		
		print(blob, end="")

	print(" \\\\")

# Write CSV table to file
os.makedirs("evaluation/", exist_ok=True)
csv_file = "evaluation.csv"
try:
	with open("evaluation/" + csv_file, 'w') as csvfile:
		writer = csv.DictWriter(csvfile, fieldnames=csv_headers)
		writer.writeheader()
		for data in csv_list:
			writer.writerow(data)
except IOError:
	print("I/O error")
